<?php

namespace App;

use Common_pack\ReqHeartBeat;
use MsgID;

require_once __DIR__ . '/../vendor/autoload.php';
require_once './msg.php';

class WebSocketClient 
{
    private \WebSocket\Client $socket;
    private array $handlers = [];
    private bool $isRunning = false;
    private int $retryCount = 0;
    private int $heartbeatInterval = 5; // 心跳间隔（秒）

    public function __construct(
        private string $url,
        private array $options = [
            'timeout' => 5,
            'heartbeat_interval' => 5, // 默认5秒
            'max_retries' => 10
        ]
    ) {
        $this->heartbeatInterval = $this->options['heartbeat_interval'];
        $this->connect();
    }

    private function connect(): void 
    {
        try {
            $this->socket = new \WebSocket\Client($this->url, [
                'timeout' => $this->options['timeout']
            ]);
        } catch (\Exception $e) {
            throw new \RuntimeException("连接失败: " . $e->getMessage());
        }
    }

    public function on(int $msgId, callable $handler): void 
    {
        $this->handlers[$msgId] = $handler;
    }

    public function sendBinary(int $msgId, string $data): void 
    {
        try {
            $binMsgId = pack('n', $msgId); // 2字节无符号短整型
            $binData = $binMsgId . $data;
            $this->socket->binary($binData);
        } catch (\Exception $e) {
            $this->handleDisconnect($e);
        }
    }

    public function startListening(): void 
    {
        if ($this->isRunning) {
            return;
        }

        $this->isRunning = true;

        // 启动心跳子进程
        $this->startHeartbeatProcess();

        // 主进程持续监听消息
        while ($this->isRunning) {
            try {
                $message = $this->socket->receive();
                if ($message) {
                    $this->dispatchBinaryMessage($message);
                }
                usleep(1000); // 1ms 避免CPU过高使用
            } catch (\WebSocket\ConnectionException $e) {
                error_log("ConnectionException 错误: " . $e->getMessage());
                $this->handleDisconnect($e);
            } catch (\Exception $e) {
                error_log("Exception 错误: " . $e->getMessage());
            }
        }
    }

    private function startHeartbeatProcess(): void
    {
        $pid = pcntl_fork();
        if ($pid == -1) {
            throw new \RuntimeException("无法创建心跳子进程");
        } elseif ($pid == 0) {
            // 子进程：定时发送心跳
            while ($this->isRunning) {
                $this->sendHeartbeat();
                sleep($this->heartbeatInterval);
            }
            exit(0); // 子进程退出
        }
    }

    private function dispatchBinaryMessage(string $binary): void 
    {
        if (strlen($binary) < 2) {
            error_log("无效的消息长度");
            return;
        }

        try {
            // 解析消息ID（前2字节）
            $msgId = unpack('n', substr($binary, 0, 2))[1];
            $data = substr($binary, 2);

            if (isset($this->handlers[$msgId])) {
                call_user_func($this->handlers[$msgId], $data);
            }
        } catch (\Exception $e) {
            error_log("消息处理错误: " . $e->getMessage());
        }
    }

    private function sendHeartbeat(): void 
    {
        try {
            $hearbeat = new ReqHeartBeat();
            $hearbeat->setMsg("ping");
            $binaryData = $hearbeat->serializeToString();
            $this->sendBinary(MsgID::REQHEARTBEAT, $binaryData);
        } catch (\Exception $e) {
            error_log("心跳发送失败: " . $e->getMessage());
        }
    }

    private function handleDisconnect(\Throwable $e): void 
    {
        error_log("连接断开: " . $e->getMessage());

        if ($this->retryCount >= $this->options['max_retries']) {
            $this->close();
            throw new \RuntimeException("达到最大重试次数");
        }

        $delay = min(pow(2, $this->retryCount), 30);
        error_log("{$delay}秒后重试连接...");
        sleep($delay);

        try {
            $this->connect();
            $this->retryCount++;
        } catch (\Exception $e) {
            $this->handleDisconnect($e);
        }
    }

    public function close(): void 
    {
        $this->isRunning = false;
        try {
            $this->socket->close();
        } catch (\Exception $e) {
            error_log("关闭连接错误: " . $e->getMessage());
        }
    }
}